package ex.cp;

import bayonet.math.SpecialFunctions;
import blang.core.ConstantSupplier;
import blang.core.DeboxedName;
import blang.core.IntVar;
import blang.core.Model;
import blang.core.ModelBuilder;
import blang.core.ModelComponent;
import blang.core.Param;
import blang.core.RealVar;
import blang.inits.Arg;
import blang.inits.DesignatedConstructor;
import blang.types.StaticUtils;
import blang.types.internals.IntScalar;
import blang.types.internals.RealScalar;
import ca.ubc.stat.blang.StaticJavaUtils;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Optional;
import java.util.function.Supplier;
import org.eclipse.xtext.xbase.lib.ExclusiveRange;

@SuppressWarnings("all")
public class ChallengerChangePoint implements Model {
  public static class Builder implements ModelBuilder {
    private boolean fromCommandLine = false;
    
    @Arg
    public Optional<List<RealVar>> failureParameters;
    
    public ChallengerChangePoint.Builder setFailureParameters(final List<RealVar> failureParameters) {
      // work around typeRef(..) limitation
      Optional<List<RealVar>> $generated__dummy = null;
      this.failureParameters = Optional.of(failureParameters);
      return this;
    }
    
    @Arg
    public Optional<RealVar> changePoint;
    
    public ChallengerChangePoint.Builder setChangePoint(final RealVar changePoint) {
      // work around typeRef(..) limitation
      Optional<RealVar> $generated__dummy = null;
      this.changePoint = Optional.of(changePoint);
      return this;
    }
    
    @Arg
    public Optional<IntVar> prediction;
    
    public ChallengerChangePoint.Builder setPrediction(final IntVar prediction) {
      // work around typeRef(..) limitation
      Optional<IntVar> $generated__dummy = null;
      this.prediction = Optional.of(prediction);
      return this;
    }
    
    @Arg
    public List<IntVar> incidents;
    
    private boolean incidents_initialized = false;
    
    public ChallengerChangePoint.Builder setIncidents(final List<IntVar> incidents) {
      incidents_initialized = true;
      this.incidents = incidents;
      return this;
    }
    
    @Arg
    public List<RealVar> temperatures;
    
    private boolean temperatures_initialized = false;
    
    public ChallengerChangePoint.Builder setTemperatures(final List<RealVar> temperatures) {
      temperatures_initialized = true;
      this.temperatures = temperatures;
      return this;
    }
    
    public ChallengerChangePoint build() {
      // For each optional type, either get the value, or evaluate the ?: expression
      List<RealVar> failureParameters;
      if (this.failureParameters != null && this.failureParameters.isPresent()) {
        failureParameters = this.failureParameters.get();
      } else {
        failureParameters = $generated__13();
      }
      final List<RealVar> __failureParameters = failureParameters;
      RealVar changePoint;
      if (this.changePoint != null && this.changePoint.isPresent()) {
        changePoint = this.changePoint.get();
      } else {
        changePoint = $generated__14(failureParameters);
      }
      final RealVar __changePoint = changePoint;
      IntVar prediction;
      if (this.prediction != null && this.prediction.isPresent()) {
        prediction = this.prediction.get();
      } else {
        prediction = $generated__15(failureParameters, changePoint);
      }
      final IntVar __prediction = prediction;
      if (!fromCommandLine && !incidents_initialized)
        throw new RuntimeException("Not all fields were set in the builder, e.g. missing incidents");
      final List<IntVar> __incidents = incidents;
      if (!fromCommandLine && !temperatures_initialized)
        throw new RuntimeException("Not all fields were set in the builder, e.g. missing temperatures");
      final List<RealVar> __temperatures = temperatures;
      // Build the instance after boxing params
      return new ChallengerChangePoint(
        __failureParameters, 
        __changePoint, 
        __prediction, 
        __incidents, 
        new ConstantSupplier(__temperatures)
      );
    }
  }
  
  @DesignatedConstructor
  public static ChallengerChangePoint.Builder builderFromCommandLine() {
    Builder result = new Builder();
    result.fromCommandLine = true;
    return result;
  }
  
  private final List<RealVar> failureParameters;
  
  public List<RealVar> getFailureParameters() {
    return failureParameters;
  }
  
  private final RealVar changePoint;
  
  public RealVar getChangePoint() {
    return changePoint;
  }
  
  private final IntVar prediction;
  
  public IntVar getPrediction() {
    return prediction;
  }
  
  private final List<IntVar> incidents;
  
  public List<IntVar> getIncidents() {
    return incidents;
  }
  
  @Param
  private final Supplier<List<RealVar>> $generated__temperatures;
  
  public List<RealVar> getTemperatures() {
    return $generated__temperatures.get();
  }
  
  /**
   * Utility main method for posterior inference on this model
   */
  public static void main(final String[] arguments) {
    StaticJavaUtils.callRunner(Builder.class, arguments);
  }
  
  /**
   * Auxiliary method generated to translate:
   * failureParameters
   */
  private static Iterable<RealVar> $generated__0(final List<RealVar> failureParameters, final RealVar changePoint, final IntVar prediction, final List<IntVar> incidents, final List<RealVar> temperatures) {
    return failureParameters;
  }
  
  /**
   * Auxiliary method generated to translate:
   * failureParameter
   */
  private static RealVar $generated__1(final RealVar failureParameter, final List<RealVar> failureParameters, final RealVar changePoint, final IntVar prediction, final List<IntVar> incidents, final List<RealVar> temperatures) {
    return failureParameter;
  }
  
  /**
   * Auxiliary method generated to translate:
   * 0.0
   */
  private static RealVar $generated__2() {
    return new blang.core.RealConstant(0.0);
  }
  
  public static class $generated__2_class implements Supplier<RealVar> {
    public RealVar get() {
      return $generated__2();
    }
    
    public String toString() {
      return "0.0";
    }
    
    public $generated__2_class() {
      
    }
  }
  
  /**
   * Auxiliary method generated to translate:
   * 100.0
   */
  private static RealVar $generated__3() {
    return new blang.core.RealConstant(100.0);
  }
  
  public static class $generated__3_class implements Supplier<RealVar> {
    public RealVar get() {
      return $generated__3();
    }
    
    public String toString() {
      return "100.0";
    }
    
    public $generated__3_class() {
      
    }
  }
  
  /**
   * Auxiliary method generated to translate:
   * changePoint
   */
  private static RealVar $generated__4(final List<RealVar> failureParameters, final RealVar changePoint, final IntVar prediction, final List<IntVar> incidents, final List<RealVar> temperatures) {
    return changePoint;
  }
  
  /**
   * Auxiliary method generated to translate:
   * 70.0
   */
  private static RealVar $generated__5() {
    return new blang.core.RealConstant(70.0);
  }
  
  public static class $generated__5_class implements Supplier<RealVar> {
    public RealVar get() {
      return $generated__5();
    }
    
    public String toString() {
      return "70.0";
    }
    
    public $generated__5_class() {
      
    }
  }
  
  /**
   * Auxiliary method generated to translate:
   * 100.0
   */
  private static RealVar $generated__6() {
    return new blang.core.RealConstant(100.0);
  }
  
  public static class $generated__6_class implements Supplier<RealVar> {
    public RealVar get() {
      return $generated__6();
    }
    
    public String toString() {
      return "100.0";
    }
    
    public $generated__6_class() {
      
    }
  }
  
  /**
   * Auxiliary method generated to translate:
   * 0 ..< incidents.size
   */
  private static Iterable<Integer> $generated__7(final List<RealVar> failureParameters, final RealVar changePoint, final IntVar prediction, final List<IntVar> incidents, final List<RealVar> temperatures) {
    int _size = incidents.size();
    ExclusiveRange _doubleDotLessThan = new ExclusiveRange(0, _size, true);
    return _doubleDotLessThan;
  }
  
  /**
   * Auxiliary method generated to translate:
   * temperatures.get(i)
   */
  private static RealVar $generated__8(final int i, final List<RealVar> failureParameters, final RealVar changePoint, final IntVar prediction, final List<IntVar> incidents, final List<RealVar> temperatures) {
    RealVar _get = temperatures.get(i);
    return _get;
  }
  
  /**
   * Auxiliary method generated to translate:
   * incidents.get(i)
   */
  private static IntVar $generated__9(final int i, final List<RealVar> failureParameters, final RealVar changePoint, final IntVar prediction, final List<IntVar> incidents, final List<RealVar> temperatures) {
    IntVar _get = incidents.get(i);
    return _get;
  }
  
  /**
   * Auxiliary method generated to translate:
   * logistic(failureParameters.get(if (temperature < changePoint) 0 else 1))
   */
  private static RealVar $generated__10(final RealVar temperature, final List<RealVar> failureParameters, final RealVar changePoint) {
    int _xifexpression = (int) 0;
    if (((temperature).doubleValue() < (changePoint).doubleValue())) {
      _xifexpression = 0;
    } else {
      _xifexpression = 1;
    }
    double _logistic = SpecialFunctions.logistic((failureParameters.get(_xifexpression)).doubleValue());
    return new blang.core.RealConstant(_logistic);
  }
  
  public static class $generated__10_class implements Supplier<RealVar> {
    public RealVar get() {
      return $generated__10(temperature, failureParameters, changePoint);
    }
    
    public String toString() {
      return "logistic(failureParameters.get(if (temperature < changePoint) 0 else 1))";
    }
    
    private final RealVar temperature;
    
    private final List<RealVar> failureParameters;
    
    private final RealVar changePoint;
    
    public $generated__10_class(final RealVar temperature, final List<RealVar> failureParameters, final RealVar changePoint) {
      this.temperature = temperature;
      this.failureParameters = failureParameters;
      this.changePoint = changePoint;
    }
  }
  
  /**
   * Auxiliary method generated to translate:
   * prediction
   */
  private static IntVar $generated__11(final List<RealVar> failureParameters, final RealVar changePoint, final IntVar prediction, final List<IntVar> incidents, final List<RealVar> temperatures) {
    return prediction;
  }
  
  /**
   * Auxiliary method generated to translate:
   * logistic(failureParameters.get(if (31 < changePoint) 0 else 1))
   */
  private static RealVar $generated__12(final List<RealVar> failureParameters, final RealVar changePoint) {
    int _xifexpression = (int) 0;
    if ((31 < (changePoint).doubleValue())) {
      _xifexpression = 0;
    } else {
      _xifexpression = 1;
    }
    double _logistic = SpecialFunctions.logistic((failureParameters.get(_xifexpression)).doubleValue());
    return new blang.core.RealConstant(_logistic);
  }
  
  public static class $generated__12_class implements Supplier<RealVar> {
    public RealVar get() {
      return $generated__12(failureParameters, changePoint);
    }
    
    public String toString() {
      return "logistic(failureParameters.get(if (31 < changePoint) 0 else 1))";
    }
    
    private final List<RealVar> failureParameters;
    
    private final RealVar changePoint;
    
    public $generated__12_class(final List<RealVar> failureParameters, final RealVar changePoint) {
      this.failureParameters = failureParameters;
      this.changePoint = changePoint;
    }
  }
  
  /**
   * Auxiliary method generated to translate:
   * latentRealList(2)
   */
  private static List<RealVar> $generated__13() {
    List<RealVar> _latentRealList = StaticUtils.latentRealList(2);
    return _latentRealList;
  }
  
  /**
   * Auxiliary method generated to translate:
   * latentReal
   */
  private static RealVar $generated__14(final List<RealVar> failureParameters) {
    RealScalar _latentReal = StaticUtils.latentReal();
    return _latentReal;
  }
  
  /**
   * Auxiliary method generated to translate:
   * latentInt
   */
  private static IntVar $generated__15(final List<RealVar> failureParameters, final RealVar changePoint) {
    IntScalar _latentInt = StaticUtils.latentInt();
    return _latentInt;
  }
  
  /**
   * Note: the generated code has the following properties used at runtime:
   *   - all arguments are annotated with a BlangVariable annotation
   *   - params additionally have a Param annotation
   *   - the order of the arguments is as follows:
   *     - first, all the random variables in the order they occur in the blang file
   *     - second, all the params in the order they occur in the blang file
   * 
   */
  public ChallengerChangePoint(@DeboxedName("failureParameters") final List<RealVar> failureParameters, @DeboxedName("changePoint") final RealVar changePoint, @DeboxedName("prediction") final IntVar prediction, @DeboxedName("incidents") final List<IntVar> incidents, @Param @DeboxedName("temperatures") final Supplier<List<RealVar>> $generated__temperatures) {
    this.failureParameters = failureParameters;
    this.changePoint = changePoint;
    this.prediction = prediction;
    this.incidents = incidents;
    this.$generated__temperatures = $generated__temperatures;
  }
  
  /**
   * A component can be either a distribution, support constraint, or another model  
   * which recursively defines additional components.
   */
  public Collection<ModelComponent> components() {
    ArrayList<ModelComponent> components = new ArrayList();
    
    for (RealVar failureParameter : $generated__0(failureParameters, changePoint, prediction, incidents, $generated__temperatures.get())) {
      { // Code generated by: failureParameter ~ Normal(0.0, 100.0)
        // Construction and addition of the factor/model:
        components.add(
          new blang.distributions.Normal(
            $generated__1(failureParameter, failureParameters, changePoint, prediction, incidents, $generated__temperatures.get()), 
            new $generated__2_class(), 
            new $generated__3_class()
          )
          );
      }
    }
    { // Code generated by: changePoint ~ Normal(70.0, 100.0)
      // Construction and addition of the factor/model:
      components.add(
        new blang.distributions.Normal(
          $generated__4(failureParameters, changePoint, prediction, incidents, $generated__temperatures.get()), 
          new $generated__5_class(), 
          new $generated__6_class()
        )
        );
    }
    for (int i : $generated__7(failureParameters, changePoint, prediction, incidents, $generated__temperatures.get())) {
      { // Code generated by: incidents.get(i) | RealVar temperature = temperatures.get(i), failureParameters, changePoint ~ Bernoulli(logistic(failureParameters.get(if (temperature < changePoint) 0 else 1)))
        // Required initialization:
        RealVar temperature = $generated__8(i, failureParameters, changePoint, prediction, incidents, $generated__temperatures.get());
        // Construction and addition of the factor/model:
        components.add(
          new blang.distributions.Bernoulli(
            $generated__9(i, failureParameters, changePoint, prediction, incidents, $generated__temperatures.get()), 
            new $generated__10_class(temperature, failureParameters, changePoint)
          )
          );
      }
    }
    { // Code generated by: prediction | failureParameters, changePoint ~ Bernoulli(logistic(failureParameters.get(if (31 < changePoint) 0 else 1)))
      // Construction and addition of the factor/model:
      components.add(
        new blang.distributions.Bernoulli(
          $generated__11(failureParameters, changePoint, prediction, incidents, $generated__temperatures.get()), 
          new $generated__12_class(failureParameters, changePoint)
        )
        );
    }
    
    return components;
  }
}
